@@ -40,7 +40,7 @@ class Arrow(object):
 
         - A ``tzinfo`` object.
         - A ``str`` describing a timezone, similar to 'US/Pacific', or 'Europe/Berlin'.
-        - A ``str`` in ISO-8601 style, as in '+07:00'.
+        - A ``str`` in ISO 8601 style, as in '+07:00'.
         - A ``str``, one of the following:  'local', 'utc', 'UTC'.
 
     Usage::
@@ -56,6 +56,12 @@ class Arrow(object):
     _ATTRS = ["year", "month", "day", "hour", "minute", "second", "microsecond"]
     _ATTRS_PLURAL = ["{}s".format(a) for a in _ATTRS]
     _MONTHS_PER_QUARTER = 3
+    _SECS_PER_MINUTE = float(60)
+    _SECS_PER_HOUR = float(60 * 60)
+    _SECS_PER_DAY = float(60 * 60 * 24)
+    _SECS_PER_WEEK = float(60 * 60 * 24 * 7)
+    _SECS_PER_MONTH = float(60 * 60 * 24 * 30.5)
+    _SECS_PER_YEAR = float(60 * 60 * 24 * 365.25)
 
     def __init__(
         self, year, month, day, hour=0, minute=0, second=0, microsecond=0, tzinfo=None
@@ -140,17 +146,19 @@ class Arrow(object):
 
         :param timestamp: an ``int`` or ``float`` timestamp, or a ``str`` that converts to either.
         :param tzinfo: (optional) a ``tzinfo`` object.  Defaults to local time.
-
-        Timestamps should always be UTC. If you have a non-UTC timestamp::
-
-            >>> arrow.Arrow.utcfromtimestamp(1367900664).replace(tzinfo='US/Pacific')
-            <Arrow [2013-05-07T04:24:24-07:00]>
-
         """
 
         if tzinfo is None:
             tzinfo = dateutil_tz.tzlocal()
-        timestamp = cls._get_timestamp_from_input(timestamp)
+        elif util.isstr(tzinfo):
+            tzinfo = parser.TzinfoParser.parse(tzinfo)
+
+        if not util.is_timestamp(timestamp):
+            raise ValueError(
+                "The provided timestamp '{}' is invalid.".format(timestamp)
+            )
+
+        timestamp = util.normalize_timestamp(float(timestamp))
         dt = datetime.fromtimestamp(timestamp, tzinfo)
 
         return cls(
@@ -172,7 +180,12 @@ class Arrow(object):
 
         """
 
-        timestamp = cls._get_timestamp_from_input(timestamp)
+        if not util.is_timestamp(timestamp):
+            raise ValueError(
+                "The provided timestamp '{}' is invalid.".format(timestamp)
+            )
+
+        timestamp = util.normalize_timestamp(float(timestamp))
         dt = datetime.utcfromtimestamp(timestamp)
 
         return cls(
@@ -342,7 +355,7 @@ class Arrow(object):
             )
 
     @classmethod
-    def span_range(cls, frame, start, end, tz=None, limit=None):
+    def span_range(cls, frame, start, end, tz=None, limit=None, bounds="[)"):
         """ Returns an iterator of tuples, each :class:`Arrow <arrow.arrow.Arrow>` objects,
         representing a series of timespans between two inputs.
 
@@ -352,6 +365,10 @@ class Arrow(object):
         :param tz: (optional) A :ref:`timezone expression <tz-expr>`.  Defaults to
             ``start``'s timezone, or UTC if ``start`` is naive.
         :param limit: (optional) A maximum number of tuples to return.
+        :param bounds: (optional) a ``str`` of either '()', '(]', '[)', or '[]' that specifies
+            whether to include or exclude the start and end values in each span in the range. '(' excludes
+            the start, '[' includes the start, ')' excludes the end, and ']' includes the end.
+            If the bounds are not specified, the default bound '[)' is used.
 
         **NOTE**: The ``end`` or ``limit`` must be provided.  Call with ``end`` alone to
         return the entire range.  Call with ``limit`` alone to return a maximum # of results from
@@ -390,10 +407,10 @@ class Arrow(object):
         tzinfo = cls._get_tzinfo(start.tzinfo if tz is None else tz)
         start = cls.fromdatetime(start, tzinfo).span(frame)[0]
         _range = cls.range(frame, start, end, tz, limit)
-        return (r.span(frame) for r in _range)
+        return (r.span(frame, bounds=bounds) for r in _range)
 
     @classmethod
-    def interval(cls, frame, start, end, interval=1, tz=None):
+    def interval(cls, frame, start, end, interval=1, tz=None, bounds="[)"):
         """ Returns an iterator of tuples, each :class:`Arrow <arrow.arrow.Arrow>` objects,
         representing a series of intervals between two inputs.
 
@@ -402,6 +419,10 @@ class Arrow(object):
         :param end: (optional) A datetime expression, the end of the range.
         :param interval: (optional) Time interval for the given time frame.
         :param tz: (optional) A timezone expression.  Defaults to UTC.
+        :param bounds: (optional) a ``str`` of either '()', '(]', '[)', or '[]' that specifies
+            whether to include or exclude the start and end values in the intervals. '(' excludes
+            the start, '[' includes the start, ')' excludes the end, and ']' includes the end.
+            If the bounds are not specified, the default bound '[)' is used.
 
         Supported frame values: year, quarter, month, week, day, hour, minute, second
 
@@ -414,7 +435,7 @@ class Arrow(object):
 
             - A ``tzinfo`` object.
             - A ``str`` describing a timezone, similar to 'US/Pacific', or 'Europe/Berlin'.
-            - A ``str`` in ISO-8601 style, as in '+07:00'.
+            - A ``str`` in ISO 8601 style, as in '+07:00'.
             - A ``str``, one of the following:  'local', 'utc', 'UTC'.
 
         Usage:
@@ -431,7 +452,7 @@ class Arrow(object):
         if interval < 1:
             raise ValueError("interval has to be a positive integer")
 
-        spanRange = iter(cls.span_range(frame, start, end, tz))
+        spanRange = iter(cls.span_range(frame, start, end, tz, bounds=bounds))
         while True:
             try:
                 intvlStart, intvlEnd = next(spanRange)
@@ -711,12 +732,23 @@ class Arrow(object):
             dt.tzinfo,
         )
 
-    def span(self, frame, count=1):
+    @classmethod
+    def _validate_bounds(cls, bounds):
+        if bounds != "()" and bounds != "(]" and bounds != "[)" and bounds != "[]":
+            raise AttributeError(
+                'Invalid bounds. Please select between "()", "(]", "[)", or "[]".'
+            )
+
+    def span(self, frame, count=1, bounds="[)"):
         """ Returns two new :class:`Arrow <arrow.arrow.Arrow>` objects, representing the timespan
         of the :class:`Arrow <arrow.arrow.Arrow>` object in a given timeframe.
 
         :param frame: the timeframe.  Can be any ``datetime`` property (day, hour, minute...).
         :param count: (optional) the number of frames to span.
+        :param bounds: (optional) a ``str`` of either '()', '(]', '[)', or '[]' that specifies
+            whether to include or exclude the start and end values in the span. '(' excludes
+            the start, '[' includes the start, ')' excludes the end, and ']' includes the end.
+            If the bounds are not specified, the default bound '[)' is used.
 
         Supported frame values: year, quarter, month, week, day, hour, minute, second.
 
@@ -734,8 +766,13 @@ class Arrow(object):
             >>> arrow.utcnow().span('day', count=2)
             (<Arrow [2013-05-09T00:00:00+00:00]>, <Arrow [2013-05-10T23:59:59.999999+00:00]>)
 
+            >>> arrow.utcnow().span('day', bounds='[]')
+            (<Arrow [2013-05-09T00:00:00+00:00]>, <Arrow [2013-05-10T00:00:00+00:00]>)
+
         """
 
+        self._validate_bounds(bounds)
+
         frame_absolute, frame_relative, relative_steps = self._get_frames(frame)
 
         if frame_absolute == "week":
@@ -760,11 +797,13 @@ class Arrow(object):
         elif frame_absolute == "quarter":
             floor = floor + relativedelta(months=-((self.month - 1) % 3))
 
-        ceil = (
-            floor
-            + relativedelta(**{frame_relative: count * relative_steps})
-            + relativedelta(microseconds=-1)
-        )
+        ceil = floor + relativedelta(**{frame_relative: count * relative_steps})
+
+        if bounds[0] == "(":
+            floor += relativedelta(microseconds=1)
+
+        if bounds[1] == ")":
+            ceil += relativedelta(microseconds=-1)
 
         return floor, ceil
 
@@ -835,7 +874,8 @@ class Arrow(object):
             Defaults to now in the current :class:`Arrow <arrow.arrow.Arrow>` object's timezone.
         :param locale: (optional) a ``str`` specifying a locale.  Defaults to 'en_us'.
         :param only_distance: (optional) returns only time difference eg: "11 seconds" without "in" or "ago" part.
-        :param granularity: (optional) defines the precision of the output. Set it to strings 'second', 'minute', 'hour', 'day', 'week', 'month' or 'year'.
+        :param granularity: (optional) defines the precision of the output. Set it to strings 'second', 'minute',
+                           'hour', 'day', 'week', 'month' or 'year' or a list of any combination of these strings
 
         Usage::
 
@@ -866,7 +906,15 @@ class Arrow(object):
                 dt = other.astimezone(self._datetime.tzinfo)
 
         else:
-            raise TypeError()
+            raise TypeError(
+                "Invalid 'other' argument of type '{}'. "
+                "Argument must be of type None, Arrow, or datetime.".format(
+                    type(other).__name__
+                )
+            )
+
+        if isinstance(granularity, list) and len(granularity) == 1:
+            granularity = granularity[0]
 
         delta = int(round(util.total_seconds(self._datetime - dt)))
         sign = -1 if delta < 0 else 1
@@ -898,7 +946,8 @@ class Arrow(object):
                     hours = sign * int(max(delta / 3600, 2))
                     return locale.describe("hours", hours, only_distance=only_distance)
 
-                elif diff < 129600:
+                # anything less than 48 hours should be 1 day
+                elif diff < 172800:
                     return locale.describe("day", sign, only_distance=only_distance)
                 elif diff < 554400:
                     days = sign * int(max(delta / 86400, 2))
@@ -928,23 +977,23 @@ class Arrow(object):
                     years = sign * int(max(delta / 31536000, 2))
                     return locale.describe("years", years, only_distance=only_distance)
 
-            else:
+            elif util.isstr(granularity):
                 if granularity == "second":
                     delta = sign * delta
                     if abs(delta) < 2:
                         return locale.describe("now", only_distance=only_distance)
                 elif granularity == "minute":
-                    delta = sign * delta / float(60)
+                    delta = sign * delta / self._SECS_PER_MINUTE
                 elif granularity == "hour":
-                    delta = sign * delta / float(60 * 60)
+                    delta = sign * delta / self._SECS_PER_HOUR
                 elif granularity == "day":
-                    delta = sign * delta / float(60 * 60 * 24)
+                    delta = sign * delta / self._SECS_PER_DAY
                 elif granularity == "week":
-                    delta = sign * delta / float(60 * 60 * 24 * 7)
+                    delta = sign * delta / self._SECS_PER_WEEK
                 elif granularity == "month":
-                    delta = sign * delta / float(60 * 60 * 24 * 30.5)
+                    delta = sign * delta / self._SECS_PER_MONTH
                 elif granularity == "year":
-                    delta = sign * delta / float(60 * 60 * 24 * 365.25)
+                    delta = sign * delta / self._SECS_PER_YEAR
                 else:
                     raise AttributeError(
                         "Invalid level of granularity. Please select between 'second', 'minute', 'hour', 'day', 'week', 'month' or 'year'"
@@ -953,9 +1002,59 @@ class Arrow(object):
                 if trunc(abs(delta)) != 1:
                     granularity += "s"
                 return locale.describe(granularity, delta, only_distance=only_distance)
+
+            else:
+                timeframes = []
+                if "year" in granularity:
+                    years = sign * delta / self._SECS_PER_YEAR
+                    delta %= self._SECS_PER_YEAR
+                    timeframes.append(["year", years])
+
+                if "month" in granularity:
+                    months = sign * delta / self._SECS_PER_MONTH
+                    delta %= self._SECS_PER_MONTH
+                    timeframes.append(["month", months])
+
+                if "week" in granularity:
+                    weeks = sign * delta / self._SECS_PER_WEEK
+                    delta %= self._SECS_PER_WEEK
+                    timeframes.append(["week", weeks])
+
+                if "day" in granularity:
+                    days = sign * delta / self._SECS_PER_DAY
+                    delta %= self._SECS_PER_DAY
+                    timeframes.append(["day", days])
+
+                if "hour" in granularity:
+                    hours = sign * delta / self._SECS_PER_HOUR
+                    delta %= self._SECS_PER_HOUR
+                    timeframes.append(["hour", hours])
+
+                if "minute" in granularity:
+                    minutes = sign * delta / self._SECS_PER_MINUTE
+                    delta %= self._SECS_PER_MINUTE
+                    timeframes.append(["minute", minutes])
+
+                if "second" in granularity:
+                    seconds = sign * delta
+                    timeframes.append(["second", seconds])
+
+                if len(timeframes) < len(granularity):
+                    raise AttributeError(
+                        "Invalid level of granularity. "
+                        "Please select between 'second', 'minute', 'hour', 'day', 'week', 'month' or 'year'."
+                    )
+
+                for tf in timeframes:
+                    # Make granularity plural if the delta is not equal to 1
+                    if trunc(abs(tf[1])) != 1:
+                        tf[0] += "s"
+                return locale.describe_multi(timeframes, only_distance=only_distance)
+
         except KeyError as e:
             raise ValueError(
-                "Humanization of the {} granularity is not currently translated in the '{}' locale. Please consider making a contribution to this locale.".format(
+                "Humanization of the {} granularity is not currently translated in the '{}' locale. "
+                "Please consider making a contribution to this locale.".format(
                     e, locale_name
                 )
             )
@@ -992,10 +1091,7 @@ class Arrow(object):
 
         """
 
-        if bounds != "()" and bounds != "(]" and bounds != "[)" and bounds != "[]":
-            raise AttributeError(
-                'Invalid bounds. Please select between "()", "(]", "[)", or "[]".'
-            )
+        self._validate_bounds(bounds)
 
         if not isinstance(start, Arrow):
             raise TypeError(
@@ -1334,19 +1430,17 @@ class Arrow(object):
 
     @classmethod
     def _get_datetime(cls, expr):
-
+        """Get datetime object for a specified expression."""
         if isinstance(expr, Arrow):
             return expr.datetime
-
-        if isinstance(expr, datetime):
+        elif isinstance(expr, datetime):
             return expr
-
-        try:
-            expr = float(expr)
-            return cls.utcfromtimestamp(expr).datetime
-        except Exception:
+        elif util.is_timestamp(expr):
+            timestamp = float(expr)
+            return cls.utcfromtimestamp(timestamp).datetime
+        else:
             raise ValueError(
-                "'{}' not recognized as a timestamp or datetime".format(expr)
+                "'{}' not recognized as a datetime or timestamp.".format(expr)
             )
 
     @classmethod
@@ -1354,13 +1448,26 @@ class Arrow(object):
 
         if name in cls._ATTRS:
             return name, "{}s".format(name), 1
-
+        elif name[-1] == "s" and name[:-1] in cls._ATTRS:
+            return name[:-1], name, 1
         elif name in ["week", "weeks"]:
             return "week", "weeks", 1
         elif name in ["quarter", "quarters"]:
             return "quarter", "months", 3
 
-        supported = ", ".join(cls._ATTRS + ["week", "weeks"] + ["quarter", "quarters"])
+        supported = ", ".join(
+            [
+                "year(s)",
+                "month(s)",
+                "day(s)",
+                "hour(s)",
+                "minute(s)",
+                "second(s)",
+                "microsecond(s)",
+                "week(s)",
+                "quarter(s)",
+            ]
+        )
         raise AttributeError(
             "range/span over frame {} not supported. Supported frames: {}".format(
                 name, supported
@@ -1382,14 +1489,6 @@ class Arrow(object):
                 return end, sys.maxsize
             return end, limit
 
-    @staticmethod
-    def _get_timestamp_from_input(timestamp):
-
-        try:
-            return float(timestamp)
-        except Exception:
-            raise ValueError("cannot parse '{}' as a timestamp".format(timestamp))
-
 
 Arrow.min = Arrow.fromdatetime(datetime.min)
 Arrow.max = Arrow.fromdatetime(datetime.max)
