@@ -22,6 +22,21 @@ def get_locale(name):
     return locale_cls()
 
 
+def get_locale_by_class_name(name):
+    """Returns an appropriate :class:`Locale <arrow.locales.Locale>`
+    corresponding to an locale class name.
+
+    :param name: the name of the locale class.
+
+    """
+    locale_cls = globals().get(name)
+
+    if locale_cls is None:
+        raise ValueError("Unsupported locale '{}'".format(name))
+
+    return locale_cls()
+
+
 # base locale type.
 
 
@@ -32,6 +47,7 @@ class Locale(object):
 
     timeframes = {
         "now": "",
+        "second": "",
         "seconds": "",
         "minute": "",
         "minutes": "",
@@ -51,6 +67,7 @@ class Locale(object):
 
     past = None
     future = None
+    and_word = None
 
     month_names = []
     month_abbreviations = []
@@ -78,6 +95,26 @@ class Locale(object):
 
         return humanized
 
+    def describe_multi(self, timeframes, only_distance=False):
+        """ Describes a delta within multiple timeframes in plain language.
+
+        :param timeframes: a list of string, quantity pairs each representing a timeframe and delta.
+        :param only_distance: return only distance eg: "2 hours and 11 seconds" without "in" or "ago" keywords
+        """
+
+        humanized = ""
+        for index, (timeframe, delta) in enumerate(timeframes):
+            humanized += self._format_timeframe(timeframe, delta)
+            if index == len(timeframes) - 2 and self.and_word:
+                humanized += " " + self.and_word + " "
+            elif index < len(timeframes) - 1:
+                humanized += " "
+
+        if not only_distance:
+            humanized = self._format_relative(humanized, timeframe, delta)
+
+        return humanized
+
     def day_name(self, day):
         """ Returns the day name for a specified day of the week.
 
@@ -200,10 +237,12 @@ class EnglishLocale(Locale):
 
     past = "{0} ago"
     future = "in {0}"
+    and_word = "and"
 
     timeframes = {
         "now": "just now",
-        "seconds": "seconds",
+        "second": "a second",
+        "seconds": "{0} seconds",
         "minute": "a minute",
         "minutes": "{0} minutes",
         "hour": "an hour",
@@ -295,16 +334,20 @@ class ItalianLocale(Locale):
     names = ["it", "it_it"]
     past = "{0} fa"
     future = "tra {0}"
+    and_word = "e"
 
     timeframes = {
         "now": "adesso",
-        "seconds": "qualche secondo",
+        "second": "un secondo",
+        "seconds": "{0} qualche secondo",
         "minute": "un minuto",
         "minutes": "{0} minuti",
         "hour": "un'ora",
         "hours": "{0} ore",
         "day": "un giorno",
         "days": "{0} giorni",
+        "week": "una settimana,",
+        "weeks": "{0} settimane",
         "month": "un mese",
         "months": "{0} mesi",
         "year": "un anno",
@@ -364,16 +407,20 @@ class SpanishLocale(Locale):
     names = ["es", "es_es"]
     past = "hace {0}"
     future = "en {0}"
+    and_word = "y"
 
     timeframes = {
         "now": "ahora",
-        "seconds": "segundos",
+        "second": "un segundo",
+        "seconds": "{0} segundos",
         "minute": "un minuto",
         "minutes": "{0} minutos",
         "hour": "una hora",
         "hours": "{0} horas",
         "day": "un día",
         "days": "{0} días",
+        "week": "una semana",
+        "weeks": "{0} semanas",
         "month": "un mes",
         "months": "{0} meses",
         "year": "un año",
@@ -431,14 +478,16 @@ class SpanishLocale(Locale):
         return "{}º".format(n)
 
 
-class FrenchLocale(Locale):
-    names = ["fr", "fr_fr"]
+class FrenchBaseLocale(Locale):
+
     past = "il y a {0}"
     future = "dans {0}"
+    and_word = "et"
 
     timeframes = {
         "now": "maintenant",
-        "seconds": "quelques secondes",
+        "second": "une seconde",
+        "seconds": "{0} quelques secondes",
         "minute": "une minute",
         "minutes": "{0} minutes",
         "hour": "une heure",
@@ -468,21 +517,6 @@ class FrenchLocale(Locale):
         "novembre",
         "décembre",
     ]
-    month_abbreviations = [
-        "",
-        "janv",
-        "févr",
-        "mars",
-        "avr",
-        "mai",
-        "juin",
-        "juil",
-        "août",
-        "sept",
-        "oct",
-        "nov",
-        "déc",
-    ]
 
     day_names = [
         "",
@@ -506,16 +540,60 @@ class FrenchLocale(Locale):
         return "{}e".format(n)
 
 
+class FrenchLocale(FrenchBaseLocale, Locale):
+
+    names = ["fr", "fr_fr"]
+
+    month_abbreviations = [
+        "",
+        "janv",
+        "févr",
+        "mars",
+        "avr",
+        "mai",
+        "juin",
+        "juil",
+        "août",
+        "sept",
+        "oct",
+        "nov",
+        "déc",
+    ]
+
+
+class FrenchCanadianLocale(FrenchBaseLocale, Locale):
+
+    names = ["fr_ca"]
+
+    month_abbreviations = [
+        "",
+        "janv",
+        "févr",
+        "mars",
+        "avr",
+        "mai",
+        "juin",
+        "juill",
+        "août",
+        "sept",
+        "oct",
+        "nov",
+        "déc",
+    ]
+
+
 class GreekLocale(Locale):
 
     names = ["el", "el_gr"]
 
     past = "{0} πριν"
     future = "σε {0}"
+    and_word = "και"
 
     timeframes = {
         "now": "τώρα",
-        "seconds": "δευτερόλεπτα",
+        "second": "ένα δεύτερο",
+        "seconds": "{0} δευτερόλεπτα",
         "minute": "ένα λεπτό",
         "minutes": "{0} λεπτά",
         "hour": "μία ώρα",
@@ -581,7 +659,8 @@ class JapaneseLocale(Locale):
 
     timeframes = {
         "now": "現在",
-        "seconds": "数秒",
+        "second": "二番目の",
+        "seconds": "{0}数秒",
         "minute": "1分",
         "minutes": "{0}分",
         "hour": "1時間",
@@ -637,16 +716,20 @@ class SwedishLocale(Locale):
 
     past = "för {0} sen"
     future = "om {0}"
+    and_word = "och"
 
     timeframes = {
         "now": "just nu",
-        "seconds": "några sekunder",
+        "second": "en sekund",
+        "seconds": "{0} några sekunder",
         "minute": "en minut",
         "minutes": "{0} minuter",
         "hour": "en timme",
         "hours": "{0} timmar",
         "day": "en dag",
         "days": "{0} dagar",
+        "week": "en vecka",
+        "weeks": "{0} veckor",
         "month": "en månad",
         "months": "{0} månader",
         "year": "ett år",
@@ -709,7 +792,8 @@ class FinnishLocale(Locale):
 
     timeframes = {
         "now": ["juuri nyt", "juuri nyt"],
-        "seconds": ["muutama sekunti", "muutaman sekunnin"],
+        "second": ["sekunti", "sekunti"],
+        "seconds": ["{0} muutama sekunti", "{0} muutaman sekunnin"],
         "minute": ["minuutti", "minuutin"],
         "minutes": ["{0} minuuttia", "{0} minuutin"],
         "hour": ["tunti", "tunnin"],
@@ -796,13 +880,16 @@ class ChineseCNLocale(Locale):
 
     timeframes = {
         "now": "刚才",
-        "seconds": "几秒",
+        "second": "一秒",
+        "seconds": "{0}秒",
         "minute": "1分钟",
         "minutes": "{0}分钟",
         "hour": "1小时",
         "hours": "{0}小时",
         "day": "1天",
         "days": "{0}天",
+        "week": "一周",
+        "weeks": "{0}周",
         "month": "1个月",
         "months": "{0}个月",
         "year": "1年",
@@ -850,16 +937,20 @@ class ChineseTWLocale(Locale):
 
     past = "{0}前"
     future = "{0}後"
+    and_word = "和"
 
     timeframes = {
         "now": "剛才",
-        "seconds": "幾秒",
+        "second": "1秒",
+        "seconds": "{0}秒",
         "minute": "1分鐘",
         "minutes": "{0}分鐘",
         "hour": "1小時",
         "hours": "{0}小時",
         "day": "1天",
         "days": "{0}天",
+        "week": "1週",
+        "weeks": "{0}週",
         "month": "1個月",
         "months": "{0}個月",
         "year": "1年",
@@ -897,7 +988,67 @@ class ChineseTWLocale(Locale):
         "12",
     ]
 
-    day_names = ["", "周一", "周二", "周三", "周四", "周五", "周六", "周日"]
+    day_names = ["", "週一", "週二", "週三", "週四", "週五", "週六", "週日"]
+    day_abbreviations = ["", "一", "二", "三", "四", "五", "六", "日"]
+
+
+class HongKongLocale(Locale):
+
+    names = ["zh_hk"]
+
+    past = "{0}前"
+    future = "{0}後"
+
+    timeframes = {
+        "now": "剛才",
+        "second": "1秒",
+        "seconds": "{0}秒",
+        "minute": "1分鐘",
+        "minutes": "{0}分鐘",
+        "hour": "1小時",
+        "hours": "{0}小時",
+        "day": "1天",
+        "days": "{0}天",
+        "week": "1星期",
+        "weeks": "{0}星期",
+        "month": "1個月",
+        "months": "{0}個月",
+        "year": "1年",
+        "years": "{0}年",
+    }
+
+    month_names = [
+        "",
+        "1月",
+        "2月",
+        "3月",
+        "4月",
+        "5月",
+        "6月",
+        "7月",
+        "8月",
+        "9月",
+        "10月",
+        "11月",
+        "12月",
+    ]
+    month_abbreviations = [
+        "",
+        " 1",
+        " 2",
+        " 3",
+        " 4",
+        " 5",
+        " 6",
+        " 7",
+        " 8",
+        " 9",
+        "10",
+        "11",
+        "12",
+    ]
+
+    day_names = ["", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六", "星期日"]
     day_abbreviations = ["", "一", "二", "三", "四", "五", "六", "日"]
 
 
@@ -910,19 +1061,34 @@ class KoreanLocale(Locale):
 
     timeframes = {
         "now": "지금",
-        "seconds": "몇 초",
+        "second": "1초",
+        "seconds": "{0}초",
         "minute": "1분",
         "minutes": "{0}분",
-        "hour": "1시간",
+        "hour": "한시간",
         "hours": "{0}시간",
-        "day": "1일",
+        "day": "하루",
         "days": "{0}일",
-        "month": "1개월",
+        "week": "1주",
+        "weeks": "{0}주",
+        "month": "한달",
         "months": "{0}개월",
         "year": "1년",
         "years": "{0}년",
     }
 
+    special_dayframes = {
+        -3: "그끄제",
+        -2: "그제",
+        -1: "어제",
+        1: "내일",
+        2: "모레",
+        3: "글피",
+        4: "그글피",
+    }
+
+    special_yearframes = {-2: "제작년", -1: "작년", 1: "내년", 2: "내후년"}
+
     month_names = [
         "",
         "1월",
@@ -957,6 +1123,24 @@ class KoreanLocale(Locale):
     day_names = ["", "월요일", "화요일", "수요일", "목요일", "금요일", "토요일", "일요일"]
     day_abbreviations = ["", "월", "화", "수", "목", "금", "토", "일"]
 
+    def _ordinal_number(self, n):
+        ordinals = ["0", "첫", "두", "세", "네", "다섯", "여섯", "일곱", "여덟", "아홉", "열"]
+        if n < len(ordinals):
+            return "{}번째".format(ordinals[n])
+        return "{}번째".format(n)
+
+    def _format_relative(self, humanized, timeframe, delta):
+        if timeframe in ("day", "days"):
+            special = self.special_dayframes.get(delta)
+            if special:
+                return special
+        elif timeframe in ("year", "years"):
+            special = self.special_yearframes.get(delta)
+            if special:
+                return special
+
+        return super(KoreanLocale, self)._format_relative(humanized, timeframe, delta)
+
 
 # derived locale types & implementations.
 class DutchLocale(Locale):
@@ -968,13 +1152,16 @@ class DutchLocale(Locale):
 
     timeframes = {
         "now": "nu",
-        "seconds": "seconden",
+        "second": "een seconde",
+        "seconds": "{0} seconden",
         "minute": "een minuut",
         "minutes": "{0} minuten",
         "hour": "een uur",
         "hours": "{0} uur",
         "day": "een dag",
         "days": "{0} dagen",
+        "week": "een week",
+        "weeks": "{0} weken",
         "month": "een maand",
         "months": "{0} maanden",
         "year": "een jaar",
@@ -1054,7 +1241,8 @@ class BelarusianLocale(SlavicBaseLocale):
 
     timeframes = {
         "now": "зараз",
-        "seconds": "некалькі секунд",
+        "second": "секунду",
+        "seconds": "{0} некалькі секунд",
         "minute": "хвіліну",
         "minutes": ["{0} хвіліну", "{0} хвіліны", "{0} хвілін"],
         "hour": "гадзіну",
@@ -1118,19 +1306,24 @@ class PolishLocale(SlavicBaseLocale):
     past = "{0} temu"
     future = "za {0}"
 
+    # The nouns should be in genitive case (Polish: "dopełniacz")
+    # in order to correctly form `past` & `future` expressions.
     timeframes = {
         "now": "teraz",
-        "seconds": "kilka sekund",
+        "second": "sekundę",
+        "seconds": ["{0} sekund", "{0} sekundy", "{0} sekund"],
         "minute": "minutę",
         "minutes": ["{0} minut", "{0} minuty", "{0} minut"],
-        "hour": "godzina",
+        "hour": "godzinę",
         "hours": ["{0} godzin", "{0} godziny", "{0} godzin"],
         "day": "dzień",
-        "days": ["{0} dzień", "{0} dni", "{0} dni"],
+        "days": "{0} dni",
+        "week": "tydzień",
+        "weeks": ["{0} tygodni", "{0} tygodnie", "{0} tygodni"],
         "month": "miesiąc",
-        "months": ["{0} miesiąc", "{0} miesiące", "{0} miesięcy"],
+        "months": ["{0} miesięcy", "{0} miesiące", "{0} miesięcy"],
         "year": "rok",
-        "years": ["{0} rok", "{0} lata", "{0} lat"],
+        "years": ["{0} lat", "{0} lata", "{0} lat"],
     }
 
     month_names = [
@@ -1186,7 +1379,8 @@ class RussianLocale(SlavicBaseLocale):
 
     timeframes = {
         "now": "сейчас",
-        "seconds": "несколько секунд",
+        "second": "Второй",
+        "seconds": "{0} несколько секунд",
         "minute": "минуту",
         "minutes": ["{0} минуту", "{0} минуты", "{0} минут"],
         "hour": "час",
@@ -1254,7 +1448,8 @@ class AfrikaansLocale(Locale):
 
     timeframes = {
         "now": "nou",
-        "seconds": "sekondes",
+        "second": "n sekonde",
+        "seconds": "{0} sekondes",
         "minute": "minuut",
         "minutes": "{0} minute",
         "hour": "uur",
@@ -1320,7 +1515,8 @@ class BulgarianLocale(SlavicBaseLocale):
 
     timeframes = {
         "now": "сега",
-        "seconds": "няколко секунди",
+        "second": "секунда",
+        "seconds": "{0} няколко секунди",
         "minute": "минута",
         "minutes": ["{0} минута", "{0} минути", "{0} минути"],
         "hour": "час",
@@ -1386,7 +1582,8 @@ class UkrainianLocale(SlavicBaseLocale):
 
     timeframes = {
         "now": "зараз",
-        "seconds": "кілька секунд",
+        "second": "секунда",
+        "seconds": "{0} кілька секунд",
         "minute": "хвилину",
         "minutes": ["{0} хвилину", "{0} хвилини", "{0} хвилин"],
         "hour": "годину",
@@ -1451,13 +1648,16 @@ class MacedonianLocale(SlavicBaseLocale):
 
     timeframes = {
         "now": "сега",
-        "seconds": "секунди",
+        "second": "една секунда",
+        "seconds": ["{0} секунда", "{0} секунди", "{0} секунди"],
         "minute": "една минута",
         "minutes": ["{0} минута", "{0} минути", "{0} минути"],
         "hour": "еден саат",
         "hours": ["{0} саат", "{0} саати", "{0} саати"],
         "day": "еден ден",
         "days": ["{0} ден", "{0} дена", "{0} дена"],
+        "week": "една недела",
+        "weeks": ["{0} недела", "{0} недели", "{0} недели"],
         "month": "еден месец",
         "months": ["{0} месец", "{0} месеци", "{0} месеци"],
         "year": "една година",
@@ -1483,56 +1683,60 @@ class MacedonianLocale(SlavicBaseLocale):
     ]
     month_abbreviations = [
         "",
-        "Јан.",
-        " Фев.",
-        " Мар.",
-        " Апр.",
-        " Мај",
-        " Јун.",
-        " Јул.",
-        " Авг.",
-        " Септ.",
-        " Окт.",
-        " Ноем.",
-        " Декем.",
+        "Јан",
+        "Фев",
+        "Мар",
+        "Апр",
+        "Мај",
+        "Јун",
+        "Јул",
+        "Авг",
+        "Септ",
+        "Окт",
+        "Ноем",
+        "Декем",
     ]
 
     day_names = [
         "",
         "Понеделник",
-        " Вторник",
-        " Среда",
-        " Четврток",
-        " Петок",
-        " Сабота",
-        " Недела",
+        "Вторник",
+        "Среда",
+        "Четврток",
+        "Петок",
+        "Сабота",
+        "Недела",
     ]
     day_abbreviations = [
         "",
-        "Пон.",
-        " Вт.",
-        " Сре.",
-        " Чет.",
-        " Пет.",
-        " Саб.",
-        " Нед.",
+        "Пон",
+        "Вт",
+        "Сре",
+        "Чет",
+        "Пет",
+        "Саб",
+        "Нед",
     ]
 
 
-class DeutschBaseLocale(Locale):
+class GermanBaseLocale(Locale):
 
     past = "vor {0}"
     future = "in {0}"
+    and_word = "und"
 
     timeframes = {
         "now": "gerade eben",
-        "seconds": "Sekunden",
+        "second": "eine Sekunde",
+        "seconds": "{0} Sekunden",
         "minute": "einer Minute",
         "minutes": "{0} Minuten",
         "hour": "einer Stunde",
         "hours": "{0} Stunden",
         "day": "einem Tag",
         "days": "{0} Tagen",
+        "week": "einer Woche",
+        "weeks": "{0} Wochen",
         "month": "einem Monat",
         "months": "{0} Monaten",
         "year": "einem Jahr",
@@ -1543,6 +1747,7 @@ class DeutschBaseLocale(Locale):
     timeframes_only_distance["minute"] = "eine Minute"
     timeframes_only_distance["hour"] = "eine Stunde"
     timeframes_only_distance["day"] = "ein Tag"
+    timeframes_only_distance["week"] = "eine Woche"
     timeframes_only_distance["month"] = "ein Monat"
     timeframes_only_distance["year"] = "ein Jahr"
 
@@ -1602,21 +1807,28 @@ class DeutschBaseLocale(Locale):
         :param only_distance: return only distance eg: "11 seconds" without "in" or "ago" keywords
         """
 
-        humanized = self.timeframes_only_distance[timeframe].format(trunc(abs(delta)))
-
         if not only_distance:
-            humanized = self._format_timeframe(timeframe, delta)
-            humanized = self._format_relative(humanized, timeframe, delta)
+            return super(GermanBaseLocale, self).describe(
+                timeframe, delta, only_distance
+            )
+
+        # German uses a different case without 'in' or 'ago'
+        humanized = self.timeframes_only_distance[timeframe].format(trunc(abs(delta)))
 
         return humanized
 
 
-class GermanLocale(DeutschBaseLocale, Locale):
+class GermanLocale(GermanBaseLocale, Locale):
 
     names = ["de", "de_de"]
 
 
-class AustrianLocale(DeutschBaseLocale, Locale):
+class SwissLocale(GermanBaseLocale, Locale):
+
+    names = ["de_ch"]
+
+
+class AustrianLocale(GermanBaseLocale, Locale):
 
     names = ["de_at"]
 
@@ -1646,7 +1858,8 @@ class NorwegianLocale(Locale):
 
     timeframes = {
         "now": "nå nettopp",
-        "seconds": "noen sekunder",
+        "second": "et sekund",
+        "seconds": "{0} noen sekunder",
         "minute": "ett minutt",
         "minutes": "{0} minutter",
         "hour": "en time",
@@ -1712,7 +1925,8 @@ class NewNorwegianLocale(Locale):
 
     timeframes = {
         "now": "no nettopp",
-        "seconds": "nokre sekund",
+        "second": "et sekund",
+        "seconds": "{0} nokre sekund",
         "minute": "ett minutt",
         "minutes": "{0} minutt",
         "hour": "ein time",
@@ -1774,75 +1988,7 @@ class PortugueseLocale(Locale):
 
     past = "há {0}"
     future = "em {0}"
-
-    timeframes = {
-        "now": "agora",
-        "second": "um segundo",
-        "seconds": "{0} segundos",
-        "minute": "um minuto",
-        "minutes": "{0} minutos",
-        "hour": "uma hora",
-        "hours": "{0} horas",
-        "day": "um dia",
-        "days": "{0} dias",
-        "week": "uma semana",
-        "weeks": "{0} semanas",
-        "month": "um mês",
-        "months": "{0} meses",
-        "year": "um ano",
-        "years": "{0} anos",
-    }
-
-    month_names = [
-        "",
-        "janeiro",
-        "fevereiro",
-        "março",
-        "abril",
-        "maio",
-        "junho",
-        "julho",
-        "agosto",
-        "setembro",
-        "outubro",
-        "novembro",
-        "dezembro",
-    ]
-    month_abbreviations = [
-        "",
-        "jan",
-        "fev",
-        "mar",
-        "abr",
-        "maio",
-        "jun",
-        "jul",
-        "ago",
-        "set",
-        "out",
-        "nov",
-        "dez",
-    ]
-
-    day_names = [
-        "",
-        "segunda-feira",
-        "terça-feira",
-        "quarta-feira",
-        "quinta-feira",
-        "sexta-feira",
-        "sábado",
-        "domingo",
-    ]
-    day_abbreviations = ["", "seg", "ter", "qua", "qui", "sex", "sab", "dom"]
-
-
-class BrazilianPortugueseLocale(PortugueseLocale):
-    names = ["pt_br"]
-
-    past = "faz {0}"
-
-    future = "em {0}"
+    and_word = "e"
 
     timeframes = {
         "now": "agora",
@@ -1906,6 +2052,12 @@ class BrazilianPortugueseLocale(PortugueseLocale):
     day_abbreviations = ["", "Seg", "Ter", "Qua", "Qui", "Sex", "Sab", "Dom"]
 
 
+class BrazilianPortugueseLocale(PortugueseLocale):
+    names = ["pt_br"]
+
+    past = "faz {0}"
+
+
 class TagalogLocale(Locale):
 
     names = ["tl", "tl_ph"]
@@ -1915,7 +2067,8 @@ class TagalogLocale(Locale):
 
     timeframes = {
         "now": "ngayon lang",
-        "seconds": "segundo",
+        "second": "isang segundo",
+        "seconds": "{0} segundo",
         "minute": "isang minuto",
         "minutes": "{0} minuto",
         "hour": "isang oras",
@@ -1984,13 +2137,16 @@ class VietnameseLocale(Locale):
 
     timeframes = {
         "now": "hiện tại",
-        "seconds": "giây",
+        "second": "một giây",
+        "seconds": "{0} giây",
         "minute": "một phút",
         "minutes": "{0} phút",
         "hour": "một giờ",
         "hours": "{0} giờ",
         "day": "một ngày",
         "days": "{0} ngày",
+        "week": "một tuần",
+        "weeks": "{0} tuần",
         "month": "một tháng",
         "months": "{0} tháng",
         "year": "một năm",
@@ -2050,7 +2206,8 @@ class TurkishLocale(Locale):
 
     timeframes = {
         "now": "şimdi",
-        "seconds": "saniye",
+        "second": "bir saniye",
+        "seconds": "{0} saniye",
         "minute": "bir dakika",
         "minutes": "{0} dakika",
         "hour": "bir saat",
@@ -2116,7 +2273,8 @@ class AzerbaijaniLocale(Locale):
 
     timeframes = {
         "now": "indi",
-        "seconds": "saniyə",
+        "second": "saniyə",
+        "seconds": "{0} saniyə",
         "minute": "bir dəqiqə",
         "minutes": "{0} dəqiqə",
         "hour": "bir saat",
@@ -2200,6 +2358,7 @@ class ArabicLocale(Locale):
 
     timeframes = {
         "now": "الآن",
+        "second": "ثانية",
         "seconds": {"double": "ثانيتين", "ten": "{0} ثوان", "higher": "{0} ثانية"},
         "minute": "دقيقة",
         "minutes": {"double": "دقيقتين", "ten": "{0} دقائق", "higher": "{0} دقيقة"},
@@ -2424,7 +2583,8 @@ class IcelandicLocale(Locale):
 
     timeframes = {
         "now": "rétt í þessu",
-        "seconds": ("nokkrum sekúndum", "nokkrar sekúndur"),
+        "second": ("sekúndu", "sekúndu"),
+        "seconds": ("{0} nokkrum sekúndum", "nokkrar sekúndur"),
         "minute": ("einni mínútu", "eina mínútu"),
         "minutes": ("{0} mínútum", "{0} mínútur"),
         "hour": ("einum tíma", "einn tíma"),
@@ -2489,10 +2649,12 @@ class DanishLocale(Locale):
 
     past = "for {0} siden"
     future = "efter {0}"
+    and_word = "og"
 
     timeframes = {
         "now": "lige nu",
-        "seconds": "et par sekunder",
+        "second": "et sekund",
+        "seconds": "{0} et par sekunder",
         "minute": "et minut",
         "minutes": "{0} minutter",
         "hour": "en time",
@@ -2558,7 +2720,8 @@ class MalayalamLocale(Locale):
 
     timeframes = {
         "now": "ഇപ്പോൾ",
-        "seconds": "സെക്കന്റ്‌",
+        "second": "ഒരു നിമിഷം",
+        "seconds": "{0} സെക്കന്റ്‌",
         "minute": "ഒരു മിനിറ്റ്",
         "minutes": "{0} മിനിറ്റ്",
         "hour": "ഒരു മണിക്കൂർ",
@@ -2631,7 +2794,8 @@ class HindiLocale(Locale):
 
     timeframes = {
         "now": "अभी",
-        "seconds": "सेकंड्",
+        "second": "एक पल",
+        "seconds": "{0} सेकंड्",
         "minute": "एक मिनट ",
         "minutes": "{0} मिनट ",
         "hour": "एक घंटा",
@@ -2695,6 +2859,7 @@ class CzechLocale(Locale):
 
     timeframes = {
         "now": "Teď",
+        "second": {"past": "vteřina", "future": "vteřina", "zero": "vteřina"},
         "seconds": {"past": "{0} sekundami", "future": ["{0} sekundy", "{0} sekund"]},
         "minute": {"past": "minutou", "future": "minutu", "zero": "{0} minut"},
         "minutes": {"past": "{0} minutami", "future": ["{0} minuty", "{0} minut"]},
@@ -2781,13 +2946,16 @@ class SlovakLocale(Locale):
 
     timeframes = {
         "now": "Teraz",
-        "seconds": {"past": "pár sekundami", "future": ["{0} sekundy", "{0} sekúnd"]},
+        "second": {"past": "sekundou", "future": "sekundu", "zero": "{0} sekúnd"},
+        "seconds": {"past": "{0} sekundami", "future": ["{0} sekundy", "{0} sekúnd"]},
         "minute": {"past": "minútou", "future": "minútu", "zero": "{0} minút"},
         "minutes": {"past": "{0} minútami", "future": ["{0} minúty", "{0} minút"]},
         "hour": {"past": "hodinou", "future": "hodinu", "zero": "{0} hodín"},
         "hours": {"past": "{0} hodinami", "future": ["{0} hodiny", "{0} hodín"]},
         "day": {"past": "dňom", "future": "deň", "zero": "{0} dní"},
         "days": {"past": "{0} dňami", "future": ["{0} dni", "{0} dní"]},
+        "week": {"past": "týždňom", "future": "týždeň", "zero": "{0} týždňov"},
+        "weeks": {"past": "{0} týždňami", "future": ["{0} týždne", "{0} týždňov"]},
         "month": {"past": "mesiacom", "future": "mesiac", "zero": "{0} mesiacov"},
         "months": {"past": "{0} mesiacmi", "future": ["{0} mesiace", "{0} mesiacov"]},
         "year": {"past": "rokom", "future": "rok", "zero": "{0} rokov"},
@@ -2796,6 +2964,7 @@ class SlovakLocale(Locale):
 
     past = "Pred {0}"
     future = "O {0}"
+    and_word = "a"
 
     month_names = [
         "",
@@ -2871,7 +3040,8 @@ class FarsiLocale(Locale):
 
     timeframes = {
         "now": "اکنون",
-        "seconds": "ثانیه",
+        "second": "یک لحظه",
+        "seconds": "{0} ثانیه",
         "minute": "یک دقیقه",
         "minutes": "{0} دقیقه",
         "hour": "یک ساعت",
@@ -2941,10 +3111,12 @@ class HebrewLocale(Locale):
 
     past = "לפני {0}"
     future = "בעוד {0}"
+    and_word = "ו"
 
     timeframes = {
         "now": "הרגע",
-        "seconds": "שניות",
+        "second": "שנייה",
+        "seconds": "{0} שניות",
         "minute": "דקה",
         "minutes": "{0} דקות",
         "hour": "שעה",
@@ -2953,6 +3125,9 @@ class HebrewLocale(Locale):
         "day": "יום",
         "days": "{0} ימים",
         "2-days": "יומיים",
+        "week": "שבוע",
+        "weeks": "{0} שבועות",
+        "2-weeks": "שבועיים",
         "month": "חודש",
         "months": "{0} חודשים",
         "2-months": "חודשיים",
@@ -3005,10 +3180,41 @@ class HebrewLocale(Locale):
     def _format_timeframe(self, timeframe, delta):
         """Hebrew couple of <timeframe> aware"""
         couple = "2-{}".format(timeframe)
+        single = timeframe.rstrip("s")
         if abs(delta) == 2 and couple in self.timeframes:
-            return self.timeframes[couple].format(abs(delta))
+            key = couple
+        elif abs(delta) == 1 and single in self.timeframes:
+            key = single
         else:
-            return self.timeframes[timeframe].format(abs(delta))
+            key = timeframe
+
+        return self.timeframes[key].format(trunc(abs(delta)))
+
+    def describe_multi(self, timeframes, only_distance=False):
+        """ Describes a delta within multiple timeframes in plain language.
+        In Hebrew, the and word behaves a bit differently.
+
+        :param timeframes: a list of string, quantity pairs each representing a timeframe and delta.
+        :param only_distance: return only distance eg: "2 hours and 11 seconds" without "in" or "ago" keywords
+        """
+
+        humanized = ""
+        for index, (timeframe, delta) in enumerate(timeframes):
+            last_humanized = self._format_timeframe(timeframe, delta)
+            if index == 0:
+                humanized = last_humanized
+            elif index == len(timeframes) - 1:  # Must have at least 2 items
+                humanized += " " + self.and_word
+                if last_humanized[0].isdecimal():
+                    humanized += "־"
+                humanized += last_humanized
+            else:  # Don't add for the last one
+                humanized += ", " + last_humanized
+
+        if not only_distance:
+            humanized = self._format_relative(humanized, timeframe, delta)
+
+        return humanized
 
 
 class MarathiLocale(Locale):
@@ -3020,7 +3226,8 @@ class MarathiLocale(Locale):
 
     timeframes = {
         "now": "सद्य",
-        "seconds": "सेकंद",
+        "second": "एक सेकंद",
+        "seconds": "{0} सेकंद",
         "minute": "एक मिनिट ",
         "minutes": "{0} मिनिट ",
         "hour": "एक तास",
@@ -3095,10 +3302,12 @@ class CatalanLocale(Locale):
     names = ["ca", "ca_es", "ca_ad", "ca_fr", "ca_it"]
     past = "Fa {0}"
     future = "En {0}"
+    and_word = "i"
 
     timeframes = {
         "now": "Ara mateix",
-        "seconds": "segons",
+        "second": "un segon",
+        "seconds": "{0} segons",
         "minute": "1 minut",
         "minutes": "{0} minuts",
         "hour": "una hora",
@@ -3170,7 +3379,8 @@ class BasqueLocale(Locale):
 
     timeframes = {
         "now": "Orain",
-        "seconds": "segundu",
+        "second": "segundo bat",
+        "seconds": "{0} segundu",
         "minute": "minutu bat",
         "minutes": "{0} minutu",
         "hour": "ordu bat",
@@ -3235,7 +3445,8 @@ class HungarianLocale(Locale):
 
     timeframes = {
         "now": "éppen most",
-        "seconds": {"past": "másodpercekkel", "future": "pár másodperc"},
+        "second": {"past": "egy második", "future": "egy második"},
+        "seconds": {"past": "{0} másodpercekkel", "future": "{0} pár másodperc"},
         "minute": {"past": "egy perccel", "future": "egy perc"},
         "minutes": {"past": "{0} perccel", "future": "{0} perc"},
         "hour": {"past": "egy órával", "future": "egy óra"},
@@ -3312,7 +3523,8 @@ class EsperantoLocale(Locale):
 
     timeframes = {
         "now": "nun",
-        "seconds": "kelkaj sekundoj",
+        "second": "sekundo",
+        "seconds": "{0} kelkaj sekundoj",
         "minute": "unu minuto",
         "minutes": "{0} minutoj",
         "hour": "un horo",
@@ -3385,7 +3597,8 @@ class ThaiLocale(Locale):
 
     timeframes = {
         "now": "ขณะนี้",
-        "seconds": "ไม่กี่วินาที",
+        "second": "วินาที",
+        "seconds": "{0} ไม่กี่วินาที",
         "minute": "1 นาที",
         "minutes": "{0} นาที",
         "hour": "1 ชั่วโมง",
@@ -3465,7 +3678,8 @@ class BengaliLocale(Locale):
 
     timeframes = {
         "now": "এখন",
-        "seconds": "সেকেন্ড",
+        "second": "একটি দ্বিতীয়",
+        "seconds": "{0} সেকেন্ড",
         "minute": "এক মিনিট",
         "minutes": "{0} মিনিট",
         "hour": "এক ঘণ্টা",
@@ -3545,7 +3759,8 @@ class RomanshLocale(Locale):
 
     timeframes = {
         "now": "en quest mument",
-        "seconds": "secundas",
+        "second": "in secunda",
+        "seconds": "{0} secundas",
         "minute": "ina minuta",
         "minutes": "{0} minutas",
         "hour": "in'ura",
@@ -3604,85 +3819,17 @@ class RomanshLocale(Locale):
     day_abbreviations = ["", "gli", "ma", "me", "gie", "ve", "so", "du"]
 
 
-class SwissLocale(Locale):
-
-    names = ["de", "de_ch"]
-
-    past = "vor {0}"
-    future = "in {0}"
-
-    timeframes = {
-        "now": "gerade eben",
-        "seconds": "Sekunden",
-        "minute": "einer Minute",
-        "minutes": "{0} Minuten",
-        "hour": "einer Stunde",
-        "hours": "{0} Stunden",
-        "day": "einem Tag",
-        "days": "{0} Tagen",
-        "week": "einer Woche",
-        "weeks": "{0} Wochen",
-        "month": "einem Monat",
-        "months": "{0} Monaten",
-        "year": "einem Jahr",
-        "years": "{0} Jahren",
-    }
-
-    month_names = [
-        "",
-        "Januar",
-        "Februar",
-        "März",
-        "April",
-        "Mai",
-        "Juni",
-        "Juli",
-        "August",
-        "September",
-        "Oktober",
-        "November",
-        "Dezember",
-    ]
-
-    month_abbreviations = [
-        "",
-        "Jan",
-        "Feb",
-        "Mär",
-        "Apr",
-        "Mai",
-        "Jun",
-        "Jul",
-        "Aug",
-        "Sep",
-        "Okt",
-        "Nov",
-        "Dez",
-    ]
-
-    day_names = [
-        "",
-        "Montag",
-        "Dienstag",
-        "Mittwoch",
-        "Donnerstag",
-        "Freitag",
-        "Samstag",
-        "Sonntag",
-    ]
-
-    day_abbreviations = ["", "Mo", "Di", "Mi", "Do", "Fr", "Sa", "So"]
-
-
 class RomanianLocale(Locale):
     names = ["ro", "ro_ro"]
 
     past = "{0} în urmă"
     future = "peste {0}"
+    and_word = "și"
 
     timeframes = {
         "now": "acum",
-        "seconds": "câteva secunde",
+        "second": "o secunda",
+        "seconds": "{0} câteva secunde",
         "minute": "un minut",
         "minutes": "{0} minute",
         "hour": "o oră",
@@ -3744,10 +3891,12 @@ class SlovenianLocale(Locale):
 
     past = "pred {0}"
     future = "čez {0}"
+    and_word = "in"
 
     timeframes = {
         "now": "zdaj",
-        "seconds": "sekund",
+        "second": "sekundo",
+        "seconds": "{0} sekund",
         "minute": "minuta",
         "minutes": "{0} minutami",
         "hour": "uro",
@@ -3814,10 +3963,12 @@ class IndonesianLocale(Locale):
 
     past = "{0} yang lalu"
     future = "dalam {0}"
+    and_word = "dan"
 
     timeframes = {
         "now": "baru saja",
-        "seconds": "detik",
+        "second": "1 sebentar",
+        "seconds": "{0} detik",
         "minute": "1 menit",
         "minutes": "{0} menit",
         "hour": "1 jam",
@@ -3886,7 +4037,8 @@ class NepaliLocale(Locale):
 
     timeframes = {
         "now": "अहिले",
-        "seconds": "सेकण्ड",
+        "second": "एक सेकेन्ड",
+        "seconds": "{0} सेकण्ड",
         "minute": "मिनेट",
         "minutes": "{0} मिनेट",
         "hour": "एक घण्टा",
@@ -3951,6 +4103,7 @@ class EstonianLocale(Locale):
 
     past = "{0} tagasi"
     future = "{0} pärast"
+    and_word = "ja"
 
     timeframes = {
         "now": {"past": "just nüüd", "future": "just nüüd"},
@@ -4020,4 +4173,89 @@ class EstonianLocale(Locale):
         return form.format(abs(delta))
 
 
+class SwahiliLocale(Locale):
+
+    names = [
+        "sw",
+        "sw_ke",
+        "sw_tz",
+    ]
+
+    past = "{0} iliyopita"
+    future = "muda wa {0}"
+    and_word = "na"
+
+    timeframes = {
+        "now": "sasa hivi",
+        "second": "sekunde",
+        "seconds": "sekunde {0}",
+        "minute": "dakika moja",
+        "minutes": "dakika {0}",
+        "hour": "saa moja",
+        "hours": "saa {0}",
+        "day": "siku moja",
+        "days": "siku {0}",
+        "week": "wiki moja",
+        "weeks": "wiki {0}",
+        "month": "mwezi moja",
+        "months": "miezi {0}",
+        "year": "mwaka moja",
+        "years": "miaka {0}",
+    }
+
+    meridians = {"am": "asu", "pm": "mch", "AM": "ASU", "PM": "MCH"}
+
+    month_names = [
+        "",
+        "Januari",
+        "Februari",
+        "Machi",
+        "Aprili",
+        "Mei",
+        "Juni",
+        "Julai",
+        "Agosti",
+        "Septemba",
+        "Oktoba",
+        "Novemba",
+        "Desemba",
+    ]
+    month_abbreviations = [
+        "",
+        "Jan",
+        "Feb",
+        "Mac",
+        "Apr",
+        "Mei",
+        "Jun",
+        "Jul",
+        "Ago",
+        "Sep",
+        "Okt",
+        "Nov",
+        "Des",
+    ]
+
+    day_names = [
+        "",
+        "Jumatatu",
+        "Jumanne",
+        "Jumatano",
+        "Alhamisi",
+        "Ijumaa",
+        "Jumamosi",
+        "Jumapili",
+    ]
+    day_abbreviations = [
+        "",
+        "Jumatatu",
+        "Jumanne",
+        "Jumatano",
+        "Alhamisi",
+        "Ijumaa",
+        "Jumamosi",
+        "Jumapili",
+    ]
+
+
 _locales = _map_locales()
